VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "DisplayIO"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'==================================================================
'
' Display Information Class
'
' 29 September 2001
' 05 March 2002 - Added display frequency
'
' Multi monitor systems (windows 98 and later)
' - The first parameter to EnumDisplaySettings can be
'   a display name retreived by EnumDisplayDevices
' - The dmPosition can be set for ChangeDisplaySettings.
' Both features have not been implemented (can't test it).
'
' You're free in using this software, as long as the
' orignal copyright/author information remains intact.
' In other words, respecting the person(s) who made it
' possible for you to view/learn something of this software.
'
' Copyright  2001 - Diederik van der Boor
' http://www.codingdomain.com
'
'==================================================================


'------------------------------------------------------------------
' Event called when enumerating resolutions

Public Event GotResolution(Width As Long, Height As Long, ColorDepth As Long, ColorName As String, Colors As Currency, Frequency As Long, ContinueEnum As Boolean)


'------------------------------------------------------------------
' Public Constants

Public Enum DispChangeResAction
  CDS_UPDATEREGISTRY = &H1
  CDS_TEST = &H2
  CDS_FULLSCREEN = &H4
  CDS_GLOBAL = &H8
  CDS_SET_PRIMARY = &H10
  CDS_NORESET = &H10000000
  CDS_SETRECT = &H20000000
  CDS_RESET = &H40000000
  CDS_FORCE = &H80000000
End Enum

'Return values for ChangeDisplaySettings
Public Enum DispChangeResResult
  DISP_CHANGE_SUCCESSFUL = 0
  DISP_CHANGE_RESTART = 1
  DISP_CHANGE_FAILED = -1
  DISP_CHANGE_BADMODE = -2
  DISP_CHANGE_NOTUPDATED = -3
  DISP_CHANGE_BADFLAGS = -4
  DISP_CHANGE_BADPARAM = -5
End Enum


'------------------------------------------------------------------
' API's required for display information

Private Declare Function EnumDisplaySettings Lib "user32" Alias "EnumDisplaySettingsA" (ByVal lpszDeviceName As Long, ByVal iModeNum As Long, lpDevMode As Any) As Boolean
Private Declare Function GetDeviceCaps Lib "gdi32" (ByVal hDc As Long, ByVal nIndex As Long) As Long
Private Declare Function ChangeDisplaySettings Lib "user32" Alias "ChangeDisplaySettingsA" (lpDevMode As Any, ByVal dwflags As Long) As Long

Private Const LOGPIXELSX As Long = 88
Private Const LOGPIXELSY As Long = 90
Private Const BITSPIXEL As Long = 12
Private Const HORZRES As Long = 8
Private Const VERTRES As Long = 10

Private Const CCDEVICENAME As Long = 32
Private Const CCFORMNAME As Long = 32

Private Const DM_GRAYSCALE As Long = &H1
Private Const DM_INTERLACED As Long = &H2

Private Const DM_BITSPERPEL As Long = &H40000
Private Const DM_PELSWIDTH As Long = &H80000
Private Const DM_PELSHEIGHT As Long = &H100000
Private Const DM_DISPLAYFLAGS As Long = &H200000
Private Const DM_DISPLAYFREQUENCY As Long = &H400000

Private Type DEVMODE
  dmDeviceName      As String * CCDEVICENAME
  dmSpecVersion     As Integer
  dmDriverVersion   As Integer
  dmSize            As Integer
  dmDriverExtra     As Integer
  dmFields          As Long
  dmOrientation     As Integer
  dmPaperSize       As Integer
  dmPaperLength     As Integer
  dmPaperWidth      As Integer
  dmScale           As Integer
  dmCopies          As Integer
  dmDefaultSource   As Integer
  dmPrintQuality    As Integer
  dmColor           As Integer
  dmDuplex          As Integer
  dmYResolution     As Integer
  dmTTOption        As Integer
  dmCollate         As Integer
  dmFormName        As String * CCFORMNAME
  dmUnusedPadding   As Integer
  dmBitsPerPel      As Integer
  dmPelsWidth       As Long
  dmPelsHeight      As Long
  dmDisplayFlags    As Long
  dmDisplayFrequency As Long
End Type



'------------------------------------------------------------------
' Change Settings

Public Function ChangeSettings(Width As Long, Height As Long, ColorDepth As Long, Frequency As Long, Optional Action As DispChangeResAction = CDS_TEST) As DispChangeResResult
  Dim DM As DEVMODE
  
  With DM
    .dmPelsWidth = Width
    .dmPelsHeight = Height
    .dmBitsPerPel = ColorDepth
    .dmDisplayFrequency = Frequency
    .dmFields = DM_PELSWIDTH Or DM_PELSHEIGHT Or DM_BITSPERPEL Or DM_DISPLAYFREQUENCY
    .dmSize = LenB(DM)
  End With
  
  ChangeSettings = ChangeDisplaySettings(DM, Action)
  If ChangeSettings < 0 Then Err.Raise 12345, "DisplayIO", "Could not change display resolution!"
End Function


'------------------------------------------------------------------
' Show the settings dialog

Public Sub ShowSettings()
  Call Shell("rundll32.exe shell32.dll,Control_RunDLL desk.cpl,,3", vbNormalFocus)
End Sub


'------------------------------------------------------------------
' Enumerate possible display resolutions

Public Sub EnumResultions()
  Dim ContinueEnum As Boolean
  Dim ColorName As String
  Dim DM As DEVMODE
  Dim I As Long
  
  'set the DEVMODE flags and structure size
  DM.dmFields = DM_PELSWIDTH Or DM_PELSHEIGHT Or DM_BITSPERPEL
  DM.dmSize = Len(DM)
  ContinueEnum = True
  
  'call the API to retrieve the values for the specified DM
  Do While EnumDisplaySettings(0&, I, DM) > 0 And ContinueEnum
    Select Case DM.dmBitsPerPel
      Case 4:      ColorName = "16 Colors"
      Case 8:      ColorName = "256 Color"
      Case 16:     ColorName = "High Color"
      Case 24, 32: ColorName = "True Color"
    End Select
    RaiseEvent GotResolution(DM.dmPelsWidth, DM.dmPelsHeight, CLng(DM.dmBitsPerPel), ColorName, 2 ^ DM.dmBitsPerPel, DM.dmDisplayFrequency, ContinueEnum)
    I = I + 1
  Loop
End Sub

